@@ -35,6 +35,7 @@
 #include "eggsmclient.h"
 
 #include <libxml/xmlversion.h>
+#include <libnotify/notify.h>
 
 #include <glib/gi18n.h>
 
@@ -261,8 +262,7 @@ main (int argc,
    */
   LIBXML_TEST_VERSION;
 
-  /* sets name to help matching with the .desktop file */
-  g_set_prgname ("epiphany");
+  notify_init (PACKAGE);
 
   /* If we're given -remote arguments, translate them */
   if (argc >= 2 && strcmp (argv[1], "-remote") == 0) {
@@ -364,8 +364,13 @@ main (int argc,
     exit (1);
   }
 
+  if (private_instance == TRUE && application_mode == TRUE) {
+    g_print ("Cannot use --private-instance and --application-mode at the same time\n");
+    exit (1);
+  }
+
   if (profile_directory != NULL && private_instance == FALSE && application_mode == FALSE) {
-    g_print ("--profile can only be used in combination wit h --private-instance or --application-mode\n");
+    g_print ("--profile can only be used in combination with --private-instance or --application-mode\n");
     exit (1);
   }
 
@@ -400,12 +405,6 @@ main (int argc,
   if (user_time == 0)
       user_time = slowly_and_stupidly_obtain_timestamp (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()));
 
-  /* sets the name to appear in the window list applet when grouping windows */
-  g_set_application_name (_("Web Browser"));
-
-  /* Set default window icon */
-  gtk_window_set_default_icon_name (EPHY_STOCK_EPHY);
-
   startup_error_quark = g_quark_from_static_string ("epiphany-startup-error");
 
   /* Start our services */
@@ -426,11 +425,31 @@ main (int argc,
   /* Now create the shell */
   if (private_instance)
     mode = EPHY_EMBED_SHELL_MODE_PRIVATE;
-  else if (application_mode)
+  else if (application_mode) {
+    char *app_name;
+
     mode = EPHY_EMBED_SHELL_MODE_APPLICATION;
-  else
+
+    app_name = g_strrstr (profile_directory, EPHY_WEB_APP_PREFIX);
+    if (app_name) {
+      /* Skip the 'app-' part */
+      app_name += strlen (EPHY_WEB_APP_PREFIX);
+
+      g_set_prgname (app_name);
+      g_set_application_name (app_name);
+
+      /* We need to re-set this because we have already parsed the
+       * options, which inits GTK+ and sets this as a side effect. */
+      gdk_set_program_class (app_name);
+    }
+  } else {
     mode = EPHY_EMBED_SHELL_MODE_BROWSER;
 
+    g_set_prgname ("epiphany");
+    g_set_application_name (_("Web Browser"));
+    gtk_window_set_default_icon_name (EPHY_STOCK_EPHY);
+  }
+
   _ephy_shell_create_instance (mode);
 
   startup_flags = get_startup_flags ();
@@ -447,6 +466,9 @@ main (int argc,
   /* Shutdown */
   g_object_unref (ephy_shell);
 
+  if (notify_is_initted ())
+    notify_uninit ();
+
   ephy_file_save_accels ();
   ephy_state_save ();
   ephy_settings_shutdown ();
